@@ -78,6 +78,26 @@
 #define LOCKWORD_COMPARE_AND_SWAP(addr, old_val, new_val) \
         COMPARE_AND_SWAP(addr, old_val, new_val)
 
+#define CACHE_LINE_LEN 32
+
+#define FLUSH_CACHE(addr, length)                   \
+{                                                   \
+    uintptr_t end = ((uintptr_t) addr) + length;    \
+    uintptr_t start = ((uintptr_t) addr)            \
+                      & ~(CACHE_LINE_LEN - 1);      \
+    uintptr_t i;                                    \
+                                                    \
+    for(i = start; i < end; i += CACHE_LINE_LEN)    \
+        __asm__ ("dcbst 0, %0" :: "r" (i));         \
+                                                    \
+    __asm__ ("sync");                               \
+                                                    \
+    for(i = start; i < end; i += CACHE_LINE_LEN)    \
+        __asm__ ("icbi 0, %0" :: "r" (i));          \
+                                                    \
+    __asm__ ("sync; isync");                        \
+}
+
 #define MBARRIER() __asm__ __volatile__ ("sync" ::: "memory")
 #define UNLOCK_MBARRIER() __asm__ __volatile__ ("sync" ::: "memory")
 #define JMM_LOCK_MBARRIER() __asm__ __volatile__ ("isync" ::: "memory")
