@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003, 2004, 2005, 2006, 2007
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008
  * Robert Lougher <rob@lougher.org.uk>.
  *
  * This file is part of JamVM.
@@ -773,6 +773,7 @@ extern void markBootClasses();
 extern void markLoaderClasses(Object *loader, int mark, int mark_soft_refs);
 extern void threadBootClasses();
 extern void threadLoaderClasses(Object *class_loader);
+extern void newLibraryUnloader(Object *class_loader, void *entry);
 extern void initialiseClass(InitArgs *args);
 
 /* resolve */
@@ -810,7 +811,9 @@ extern void *executeMethodList(Object *ob, Class *class, MethodBlock *mb, u8 *ar
 /* excep */
 
 extern Object *exceptionOccurred();
-extern void signalChainedException(char *excep_name, char *excep_mess, Object *cause);
+extern void signalChainedExceptionEnum(int excep_enum, char *excep_mess, Object *cause);
+extern void signalChainedExceptionName(char *excep_name, char *excep_mess, Object *cause);
+extern void signalChainedExceptionClass(Class *excep_class, char *excep_mess, Object *cause);
 extern void setException(Object *excep);
 extern void clearException();
 extern void printException();
@@ -825,7 +828,13 @@ extern void initialiseException();
     ee->exception
 
 #define signalException(excep_name, excep_mess) \
-    signalChainedException(excep_name, excep_mess, NULL)
+    signalChainedExceptionEnum(EXCEPTION_ENUM(excep_name), excep_mess, NULL)
+
+#define signalChainedException(excep_name, excep_mess, cause) \
+    signalChainedExceptionEnum(EXCEPTION_ENUM(excep_name), excep_mess, cause)
+
+#define signalExceptionClass(excep_class, excep_mess) \
+    signalChainedExceptionClass(excep_class, excep_mess, NULL)
 
 #define setStackTrace() \
     setStackTrace0(getExecEnv(), INT_MAX)
@@ -860,13 +869,20 @@ extern int utf8Len(char *utf8);
 extern int utf8Hash(char *utf8);
 extern int utf8Comp(char *utf81, char *utf82);
 extern void convertUtf8(char *utf8, unsigned short *buff);
-extern char *findUtf8String(char *string);
+extern char *findHashedUtf8(char *string, int add_if_absent);
+extern char *copyUtf8(char *string);
 extern int utf8CharLen(unsigned short *unicode, int len);
 extern char *unicode2Utf8(unsigned short *unicode, int len, char *utf8);
 extern char *slash2dots(char *utf8);
 extern char *slash2dots2buff(char *utf8, char *buff, int buff_len);
 extern void initialiseUtf8();
 
+#define findUtf8(string) \
+    findHashedUtf8(string, FALSE)
+
+#define newUtf8(string) \
+    findHashedUtf8(string, TRUE)
+
 /* Dll */
 
 extern void *resolveNativeMethod(MethodBlock *mb);
@@ -876,6 +892,7 @@ extern char *getBootDllPath();
 extern char *getDllName(char *name);
 extern void initialiseDll(InitArgs *args);
 extern uintptr_t *resolveNativeWrapper(Class *class, MethodBlock *mb, uintptr_t *ostack);
+extern void unloaderUnloadDll(uintptr_t entry);
 extern void unloadClassLoaderDlls(Object *loader);
 extern void threadLiveClassLoaderDlls();
 
@@ -891,7 +908,8 @@ extern int nativeAvailableProcessors();
 
 /* Threading */
 
-extern void initialiseMainThread(InitArgs *args);
+extern void initialiseThreadStage1(InitArgs *args);
+extern void initialiseThreadStage2(InitArgs *args);
 extern ExecEnv *getExecEnv();
 
 extern void createJavaThread(Object *jThread, long long stack_size);
@@ -939,6 +957,7 @@ extern Class *getReflectMethodClass();
 
 /* jni */
 
+extern int initJNILrefs();
 extern void initialiseJNI();
 extern void *getJNIInterface();
 extern void markJNIGlobalRefs();
@@ -985,3 +1004,7 @@ extern void jamvm_exit(int status);
 extern void freeMethodInlinedInfo(MethodBlock *mb);
 extern int  initialiseInlining(InitArgs *args);
 extern void showRelocatability();
+
+/* symbol */
+extern void initialiseSymbol();
+
