@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003, 2004, 2005, 2006, 2007
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008
  * Robert Lougher <rob@lougher.org.uk>.
  *
  * This file is part of JamVM.
@@ -22,6 +22,8 @@
 #include <stdio.h>
 #include <string.h>
 #include "jam.h"
+#include "symbol.h"
+#include "excep.h"
 
 MethodBlock *findMethod(Class *class, char *methodname, char *type) {
    ClassBlock *cb = CLASS_CB(class);
@@ -29,7 +31,7 @@ MethodBlock *findMethod(Class *class, char *methodname, char *type) {
    int i;
 
    for(i = 0; i < cb->methods_count; i++,mb++)
-       if((strcmp(mb->name, methodname) == 0) && (strcmp(mb->type, type) == 0))
+       if(mb->name == methodname && mb->type == type)
           return mb;
 
    return NULL;
@@ -45,7 +47,7 @@ FieldBlock *findField(Class *class, char *fieldname, char *type) {
     int i;
 
     for(i = 0; i < cb->fields_count; i++,fb++)
-        if(strcmp(fb->name, fieldname) == 0 && (strcmp(fb->type, type) == 0))
+        if(fb->name == fieldname && fb->type == type)
             return fb;
 
     return NULL;
@@ -116,7 +118,7 @@ retry:
                 return NULL;
 
             if(!checkClassAccess(resolved_class, class)) {
-                signalException("java/lang/IllegalAccessException", "class is not accessible");
+                signalException(java_lang_IllegalAccessError, "class is not accessible");
                 return NULL;
             }
 
@@ -169,7 +171,7 @@ retry:
                 return NULL;
 
             if(resolved_cb->access_flags & ACC_INTERFACE) {
-                signalException("java/lang/IncompatibleClassChangeError", NULL);
+                signalException(java_lang_IncompatibleClassChangeError, NULL);
                 return NULL;
             }
             
@@ -178,16 +180,17 @@ retry:
             if(mb) {
                 if((mb->access_flags & ACC_ABSTRACT) &&
                        !(resolved_cb->access_flags & ACC_ABSTRACT)) {
-                    signalException("java/lang/AbstractMethodError", methodname);
+                    signalException(java_lang_AbstractMethodError, methodname);
                     return NULL;
                 }
 
                 if(!checkMethodAccess(mb, class)) {
-                    signalException("java/lang/IllegalAccessException", "method is not accessible");
+                    signalException(java_lang_IllegalAccessError, "method is not accessible");
                     return NULL;
                 }
 
-                initClass(mb->class);
+                if(initClass(mb->class) == NULL)
+                    return NULL;
 
                 CP_TYPE(cp, cp_index) = CONSTANT_Locked;
                 MBARRIER();
@@ -195,7 +198,7 @@ retry:
                 MBARRIER();
                 CP_TYPE(cp, cp_index) = CONSTANT_Resolved;
             } else
-                signalException("java/lang/NoSuchMethodError", methodname);
+                signalException(java_lang_NoSuchMethodError, methodname);
 
             break;
         }
@@ -234,11 +237,12 @@ retry:
                 return NULL;
 
             if(!(CLASS_CB(resolved_class)->access_flags & ACC_INTERFACE)) {
-                signalException("java/lang/IncompatibleClassChangeError", NULL);
+                signalException(java_lang_IncompatibleClassChangeError, NULL);
                 return NULL;
             }
             
-            if((mb = lookupMethod(resolved_class, methodname, methodtype)) == NULL) {
+            mb = lookupMethod(resolved_class, methodname, methodtype);
+            if(mb == NULL) {
                 ClassBlock *cb = CLASS_CB(resolved_class);
                 int i;
 
@@ -255,7 +259,7 @@ retry:
                 MBARRIER();
                 CP_TYPE(cp, cp_index) = CONSTANT_Resolved;
             } else
-                signalException("java/lang/NoSuchMethodError", methodname);
+                signalException(java_lang_NoSuchMethodError, methodname);
 
             break;
         }
@@ -297,11 +301,12 @@ retry:
 
             if(fb) {
                 if(!checkFieldAccess(fb, class)) {
-                    signalException("java/lang/IllegalAccessException", "field is not accessible");
+                    signalException(java_lang_IllegalAccessError, "field is not accessible");
                     return NULL;
                 }
 
-                initClass(fb->class);
+                if(initClass(fb->class) == NULL)
+                    return NULL;
 
                 CP_TYPE(cp, cp_index) = CONSTANT_Locked;
                 MBARRIER();
@@ -309,7 +314,7 @@ retry:
                 MBARRIER();
                 CP_TYPE(cp, cp_index) = CONSTANT_Resolved;
             } else
-                signalException("java/lang/NoSuchFieldError", fieldname);
+                signalException(java_lang_NoSuchFieldError, fieldname);
 
             break;
         }
@@ -371,7 +376,7 @@ MethodBlock *lookupVirtualMethod(Object *ob, MethodBlock *mb) {
                    (mb->class != cb->imethod_table[i].interface); i++);
 
         if(i == cb->imethod_table_size) {
-            signalException("java/lang/IncompatibleClassChangeError",
+            signalException(java_lang_IncompatibleClassChangeError,
                             "unimplemented interface");
             return NULL;
         }
@@ -381,7 +386,7 @@ MethodBlock *lookupVirtualMethod(Object *ob, MethodBlock *mb) {
     mb = cb->method_table[mtbl_idx];
 
     if(mb->access_flags & ACC_ABSTRACT) {
-        signalException("java/lang/AbstractMethodError", mb->name);
+        signalException(java_lang_AbstractMethodError, mb->name);
         return NULL;
     }
 
