@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003, 2004, 2005, 2006, 2007
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008
  * Robert Lougher <rob@lougher.org.uk>.
  *
  * This file is part of JamVM.
@@ -24,6 +24,7 @@
 #include <stdlib.h>
 #include "jam.h"
 #include "hash.h"
+#include "symbol.h"
 
 #define HASHTABSZE 1<<10
 #define HASH(ptr) stringHash(ptr)
@@ -40,8 +41,6 @@ static int offset_offset;
 
 static HashTable hash_table;
 
-static int inited = FALSE;
-
 int stringHash(Object *ptr) {
     Object *array = (Object*)INST_DATA(ptr)[value_offset]; 
     int len = INST_DATA(ptr)[count_offset];
@@ -82,9 +81,6 @@ Object *createString(char *utf8) {
     Object *array;
     Object *ob;
 
-    if(!inited)
-        initialiseString();
-
     if((array = allocTypeArray(T_CHAR, len)) == NULL ||
        (ob = allocObject(string_class)) == NULL)
         return NULL;
@@ -169,31 +165,29 @@ char *String2Cstr(Object *string) {
 }
 
 void initialiseString() {
-    if(!inited) {
-        FieldBlock *count, *value, *offset;
+    FieldBlock *count = NULL, *value, *offset;
 
-        /* As we're initialising, VM will abort if String can't be found */
-        string_class = findSystemClass0("java/lang/String");
-        registerStaticClassRef(&string_class);
+    string_class = findSystemClass0(SYMBOL(java_lang_String));
+    registerStaticClassRef(&string_class);
 
-        count = findField(string_class, "count", "I");
-        value = findField(string_class, "value", "[C");
-        offset = findField(string_class, "offset", "I");
-
-        /* findField doesn't throw an exception... */
-        if((count == NULL) || (value == NULL) || (offset == NULL)) {
-            jam_fprintf(stderr, "Error initialising VM (initialiseString)\n");
-            exitVM(1);
-        }
-
-        count_offset = count->offset;
-        value_offset = value->offset;
-        offset_offset = offset->offset;
-
-        /* Init hash table and create lock */
-        initHashTable(hash_table, HASHTABSZE, TRUE);
-        inited = TRUE;
+    if(string_class != NULL) {
+        count = findField(string_class, SYMBOL(count), SYMBOL(I));
+        value = findField(string_class, SYMBOL(value), SYMBOL(array_C));
+        offset = findField(string_class, SYMBOL(offset), SYMBOL(I));
     }
+
+    /* findField doesn't throw an exception... */
+    if((count == NULL) || (value == NULL) || (offset == NULL)) {
+        jam_fprintf(stderr, "Error initialising VM (initialiseString)\n");
+        exitVM(1);
+    }
+
+    count_offset = count->offset;
+    value_offset = value->offset;
+    offset_offset = offset->offset;
+
+    /* Init hash table and create lock */
+    initHashTable(hash_table, HASHTABSZE, TRUE);
 }
 
 #ifndef NO_JNI
